/**
 * Copyright 2020 Huawei Technologies Co., Ltd
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef GE_GRAPH_PASSES_INFERSHAPE_PASS_H_
#define GE_GRAPH_PASSES_INFERSHAPE_PASS_H_

#include "graph/passes/infer_base_pass.h"

namespace ge {
class InferShapePass : public InferBasePass {
 public:
  static void ClearContextMap();
  graphStatus Infer(NodePtr &node) override;
  bool TensorDescChanged(const GeTensorDescPtr &src, const GeTensorDescPtr &dst) override;
  graphStatus UpdateInputDescAttr(const GeTensorDescPtr &src, GeTensorDescPtr &dst, bool &changed) override;
  void AnalyzeFailedInfo(const NodePtr &node) override;

  static graphStatus InferShapeAndType(NodePtr &node);                        // temp: visible static func
  static graphStatus InferShapeAndType(NodePtr &node, bool before_subgraph);  // temp: visible static func

 private:
  graphStatus CallInferShapeFunc(NodePtr &node, Operator &op);
  graphStatus UpdatePeerInputs(NodePtr &node) override;  // only for infershape, will be deleted
  void UpdateInputOutputOriginAttr(NodePtr &node);       // only for infershape, will be deleted
  Status DoRepassForLoopNode(NodePtr &node) override;    // only for infershape, will be deleted
  Status RePassLoopNode(const NodePtr &node);            // only for infershape, will be deleted
};


class InferShapeForRunning : public InferBasePass {
 public:
  graphStatus Infer(NodePtr &node) override;
  bool TensorDescChanged(const GeTensorDescPtr &src, const GeTensorDescPtr &dst) override;

  static graphStatus InferShapeAndTypeForRunning(NodePtr &node, bool before_subgraph); // temp: visible static func

 private:
  graphStatus CallInferShapeFuncForRunning(NodePtr &node, Operator &op);
};
}  // namespace ge
#endif  // GE_GRAPH_PASSES_INFERSHAPE_PASS_H_
